
import sys,os
import globals
from threading import Lock

import copy


nodes_config={}  # key: config's id. value: config
edges_config={}

flow_nodes_config={}
data_nodes_config={}
event_nodes_config={}

dtio_nodes_config={}
dtpc_nodes_config={}
ctrl_nodes_config={}
func_nodes_config={}
evrx_nodes_config={}
evtx_nodes_config={}

nodes_link_input={}  # node 输入参数(input)中被链接的参数名。 key:node_id-->value:param_list
nodes_link_input_sourcemapping={}  # node 输入参数(input)中被链接的参数值数据来源。key:node_id1.input.param_name-->value:node_id2.output.param_name


is_active=False
is_active_lock=Lock()


def init(**kwargs):
    # ini config
    global nodes_config,edges_config

    # input config
    if "node_list" in kwargs:
        node_list=kwargs["node_list"]
        for node in node_list:
            nodes_config[node['id']]=node
    if "edge_list" in kwargs:
        edge_list=kwargs["edge_list"]
        for edge in edge_list:
            edges_config[edge['id']]=edge
    
    if "node_config" in kwargs:
        nodes_config=kwargs["node_config"]
    if "edge_config" in kwargs:
        edges_config=kwargs["edge_config"]
    
    global flow_nodes_config,data_nodes_config,event_nodes_config
    global dtio_nodes_config,dtpc_nodes_config,ctrl_nodes_config,func_nodes_config,evtx_nodes_config,evrx_nodes_config

    for key in nodes_config:
        if nodes_config[key]["node_type"]=="evrx":
            evrx_nodes_config[key]=nodes_config[key]
            event_nodes_config[key]=nodes_config[key]
        elif nodes_config[key]["node_type"]=="evtx":
            evtx_nodes_config[key]=nodes_config[key]
            event_nodes_config[key]=nodes_config[key]
        elif nodes_config[key]["node_type"]=="dtio":
            data_nodes_config[key]=nodes_config[key]
            dtio_nodes_config[key]=nodes_config[key]
        elif nodes_config[key]["node_type"]=="dtpc":
            data_nodes_config[key]=nodes_config[key]
            dtpc_nodes_config[key]=nodes_config[key]
        elif nodes_config[key]["node_type"]=="ctrl":
            flow_nodes_config[key]=nodes_config[key]
            ctrl_nodes_config[key]=nodes_config[key]
        else:
        # elif nodes_config[key]["node_type"]=="func":
            flow_nodes_config[key]=nodes_config[key]
            func_nodes_config[key]=nodes_config[key]
    
    global nodes_link_input,nodes_link_input_sourcemapping
    
    for key in edges_config:
        pre_id=str(edges_config[key]["pre_id"])
        nxt_id=str(edges_config[key]["nxt_id"])
        output_index=pre_id.find(".output.")
        input_index=nxt_id.find(".input.")
        if input_index>-1 and output_index>-1:
            nodes_link_input_sourcemapping[nxt_id]=pre_id
            node_id=nxt_id[0:input_index]
            if node_id in nodes_link_input:
                nodes_link_input[node_id].append(nxt_id[input_index+7:])
            else:
                nodes_link_input[node_id]=[nxt_id[input_index+7:]]


def start(**kwargs):
    global is_active
    with is_active_lock:
        is_active=True
    init_data()
    # start events
    for id in evrx_nodes_config:
        handle_flow_node(id)


def stop(**kwargs):
    global is_active
    with is_active_lock:
        is_active=False
    # stop events
    


def init_data(**kwargs):
    node_to_be_run=[_ for _ in data_nodes_config]
    while len(node_to_be_run)>0:
        length=len(node_to_be_run)
        for id in copy.deepcopy(node_to_be_run):
            if not is_node_linkin_data_ready(id):
                continue
            handle_data_node(id)
            node_to_be_run.remove(id)
        if length==len(node_to_be_run):
            break


def handle_flow_node(id,**kwargs):
    if id not in flow_nodes_config and id not in event_nodes_config or not is_active:
        # is not flow about
        return
    # load input params
    node_config=copy.deepcopy(nodes_config[id])
    if id in event_nodes_config:
        # handle event
        node=globals.node_center.get_or_generate_put(node_config["node_name"],node_config["node_name"])  # event node is single
        @globals.aop(id=id,name=node_config["node_name"])
        def node_run(*args,**kwargs):
            results = node.run(_config=copy.deepcopy(nodes_config[id]),*args,**kwargs)
            return results
        results=node_run(**node_config["input"])
    else:
        # handle flow
        if not is_node_linkin_data_ready(id):
            print("{}.{}: Error!!! Node input data missing!!!".format(id,node_config["node_name"]))
            return
        input_data=get_node_linkin_data(id)
        node_config["input"]={**node_config["input"],**input_data}  # Notice the order, input_data must be at the end
        # print("node_input_data",node_config["input"],input_data)
        # node start
        node=globals.node_center.get_or_generate(id,node_config["node_name"])
        @globals.aop(id=id,name=node_config["node_name"])
        def node_run(*args,**kwargs):
            results = node.run(_config=copy.deepcopy(nodes_config[id]),*args,**kwargs)
            return results
        results=node_run(**node_config["input"])
        # store node and keep alive
        if results is None or not isinstance(results,dict):
            results={}
        if "keep_alive" in results and results["keep_alive"]==True:
            globals.node_center.put(id,node)
        # store output params
        put_node_linkout_data(id,results)
        # start next node
        if  id in ctrl_nodes_config:
            return
        if "nxt_edge_id" not in node_config or len(node_config["nxt_edge_id"])<=0:
            return
        edge_id=node_config["nxt_edge_id"][0]
        if edge_id in edges_config:
            handle_flow_node(edges_config[edge_id]["nxt_id"])


def handle_data_node(id,**kwargs):
    if id not in dtpc_nodes_config and id not in dtio_nodes_config:
        # is not data about
        return
    node_config=copy.deepcopy(nodes_config[id])
    if not is_node_linkin_data_ready(id):
        return
    input_data=get_node_linkin_data(id)
    node_config["input"]={**node_config["input"],**input_data}  # Notice the order, input_data must be at the end
    # print("node_input_data",node_config["input"],input_data)
    # node start
    node=globals.node_center.get_or_generate(id,node_config["node_name"])
    @globals.aop(id=id,name=node_config["node_name"])
    def node_run(*args,**kwargs):
        results = node.run(_config=copy.deepcopy(nodes_config[id]),*args,**kwargs)
        return results
    results=node_run(**node_config["input"])
    # store node and keep alive
    if results is None or not isinstance(results,dict):
        results={}
    if "keep_alive" in results and results["keep_alive"]==True:
        globals.node_center.put(id,node)
    # store output params
    put_node_linkout_data(id,results)


def is_node_linkin_data_ready(id,**kwargs):
    if id not in nodes_link_input:
        return True
    for param_name in nodes_link_input[id]:
        data_key=nodes_link_input_sourcemapping["{}.input.{}".format(id,param_name)]
        if not globals.data_center.has(data_key):
            # Error! 缺少数据
            return False
    return True


def get_node_linkin_data(id,**kwargs):
    if id not in nodes_link_input:
        return {}
    result={}
    for param_name in nodes_link_input[id]:
        data_key=nodes_link_input_sourcemapping["{}.input.{}".format(id,param_name)]
        # print(id,param_name,data_key)
        if not globals.data_center.has(data_key):
            # Error! 缺少数据
            print("Error! 缺少数据")
            # return {}
            pass
        else:
            result[param_name]=globals.data_center.get(data_key)
    return result


def put_node_linkout_data(id,output_data,**kwargs):
    dtpc_to_be_run=[]
    if output_data is None or not isinstance(output_data,dict):
        return
    for key in output_data:
        data_id="{}.output.{}".format(id,key)
        globals.data_center.put(data_id,output_data[key])
        for edge_id in edges_config:
            edge=edges_config[edge_id]
            nxt_node_id=edge["nxt_id"][0:edge["nxt_id"].find(".input.")]
            if data_id==edge["pre_id"] and nxt_node_id in dtpc_nodes_config:
                dtpc_to_be_run.append(nxt_node_id)
    while len(dtpc_to_be_run)>0:
        length=len(dtpc_to_be_run)
        for dtpc_id in copy.deepcopy(dtpc_to_be_run):
            if not is_node_linkin_data_ready(dtpc_id):
                continue
            handle_data_node(dtpc_id)
            dtpc_to_be_run.remove(dtpc_id)
        if length==len(dtpc_to_be_run):
            break











































import globals
from threading import Thread


g_is_started=False
g_is_finished=False


def is_started():
    return g_is_started


def is_finished():
    return g_is_finished


def event_run(**kwargs):
    global g_is_started,g_is_finished
    g_is_started=True
    g_is_finished=True
    globals.engine.stop()


def run(**kwargs):
    t=Thread(target=event_run,kwargs=kwargs)
    t.start()


























import sys,os
import globals
import copy


def run(**kwargs):
    # print("control select...")
    index=0
    if "index" in kwargs:
        index=kwargs["index"]
    node_config=kwargs["_config"]
    if "nxt_edge_id" not in node_config or len(node_config["nxt_edge_id"])<=index:
        return
    edge_id=node_config["nxt_edge_id"][index]
    if edge_id in globals.engine.edges_config:
        globals.engine.handle_flow_node(globals.engine.edges_config[edge_id]["nxt_id"])
    return {}

























import sys
import os
import globals
from threading import Lock

import copy

"""
key: data id
value: data value
"""
data_dict = {}
data_dict_lock = Lock()


def init(**kwargs):
    pass


def put(key, value, **kwargs):
    with data_dict_lock:
        data_dict[key] = copy.deepcopy(value)


def get(key, **kwargs):
    ret = None
    with data_dict_lock:
        ret = copy.deepcopy(data_dict[key])
    return ret


def get_dict_by_prefix(key, **kwargs):
    tmp_dict = {}
    with data_dict_lock:
        for k in data_dict:
            if str(k).startswith(key):
                tmp_dict[k] = copy.deepcopy(data_dict[k])
    return tmp_dict


def has(key, **kwargs):
    ret = False
    with data_dict_lock:
        ret = key in data_dict
    return ret




import sys,os
import globals
import copy


def run(**kwargs):
    # print("control for...")
    node_config=kwargs["_config"]
    start_index=0
    end_index=5
    step=1
    loop_edge_id=""
    if "start_index" in kwargs:
        start_index=int(kwargs["start_index"])
    if "end_index" in kwargs:
        end_index=int(kwargs["end_index"])
    if "step" in kwargs:
        step=int(kwargs["step"])
    if "loop_edge_id" in node_config["output"]:
        loop_edge_id=node_config["output"]["loop_edge_id"]
    # find edge and cover config
    for key in globals.engine.edges_config:
        edge=globals.engine.edges_config[key]
        if edge["pre_id"]=="{}.output.loop_edge_id".format(node_config["id"]):
            loop_edge_id=key
            break
    # loop 
    loop_node_id=None
    if loop_edge_id in globals.engine.edges_config:
        loop_node_id=globals.engine.edges_config[loop_edge_id]["nxt_id"]
    for index in range(start_index,end_index,step):
        cur_node_result={"index":index}
        globals.engine.put_node_linkout_data(node_config["id"],cur_node_result)
        if loop_node_id is not None:
            globals.engine.handle_flow_node(loop_node_id)
    # loop end. flow next node
    if "nxt_edge_id" not in node_config or len(node_config["nxt_edge_id"])<=0:
        return
    edge_id=node_config["nxt_edge_id"][0]
    if edge_id in globals.engine.edges_config:
        globals.engine.handle_flow_node(globals.engine.edges_config[edge_id]["nxt_id"])
    return {}
































def run(**kwargs):
    bool=False
    string_1=""
    string_2=""
    if "bool" in kwargs:
        bool=kwargs["bool"]
    if "string_1" in kwargs:
        string_1=kwargs["string_1"]
    if "string_2" in kwargs:
        string_2=kwargs["string_2"]
    return {"result":string_1 if bool else string_2}






































import sys
import os
import globals

from functools import wraps
import os
import datetime


# config
AOP_LOG_FILE = "log/aop.log"
LOG_OPEN = True  # disable


def init(**kwargs):
    global AOP_LOG_FILE, LOG_OPEN
    if "AOP_LOG_FILE" in kwargs:
        AOP_LOG_FILE = kwargs['AOP_LOG_FILE']
    else:
        AOP_LOG_FILE = os.path.join(globals.ENV_WORK_DIR, AOP_LOG_FILE)
    if "LOG_OPEN" in kwargs:
        LOG_OPEN = kwargs['LOG_OPEN']
    if LOG_OPEN and len(AOP_LOG_FILE.split('/')) > 1:
        [path, filename] = os.path.split(AOP_LOG_FILE)
        # print(path,filename)
        if not os.path.exists(path):
            os.makedirs(path)


def log(msg):
    msg = "[{}][aop]: {}\n".format(datetime.datetime.now(), msg)
    if not LOG_OPEN:
        # print(msg,end="")
        return
    # aop log
    with open(AOP_LOG_FILE, "+a", encoding="utf-8") as f:
        f.write(msg)
        # print(msg,end="")
    # common log
    if globals.common_log:
        globals.common_log(msg=msg)


def aop(id, name):
    def c(func):
        @wraps(func)
        def b(*args, **kwargs):
            log('{}.{}  node id:     {}'.format(id, name, id))
            log('{}.{}  node name:   {}'.format(id, name, name))
            log('{}.{}  node params: args={}, kwargs={}'.format(
                id, name, args, kwargs))
            log('{}.{}  node start...'.format(id, name))
            results = func(*args, **kwargs)
            log('{}.{}  node end...'.format(id, name))
            log('{}.{}  node return  results: {}'.format(id, name, results))
            return results
        return b
    return c


# Usage:
# @aop(name=os.path.basename(__file__))
# def a(name, age, lang):
#     print('函数执行中。。。')
#     return "函数结束返回值：我是 {}, 今年{}岁, 讲{}.".format(name, age,lang), "end"

# print(a('A', 24, lang="en"))
# print(a('B', 24, lang="ch"))




import sys,os
import globals
import copy


def run(**kwargs):
    # print("control queue...")
    node_config=kwargs["_config"]
    if "nxt_edge_id" not in node_config:
        return
    for edge_id in node_config["nxt_edge_id"]:
        if edge_id in globals.engine.edges_config:
            globals.engine.handle_flow_node(globals.engine.edges_config[edge_id]["nxt_id"])
    return {}
































def run(**kwargs):
    number_1=0
    number_2=0
    if "number_1" in kwargs:
        number_1=kwargs["number_1"]
    if "number_2" in kwargs:
        number_2=kwargs["number_2"]
    return {"result":number_1>number_2}





































import sys
import os
import glob
import globals
import imp
from threading import Lock

lib_node_set = set()

"""
key: node id
value: node
"""
node_dict = {}
node_dict_lock = Lock()


def init(**kwargs):
    # init lib_node_set
    USER_LIB_PATH = 'USER_LIB_PATH'
    global lib_node_set
    import os
    lib_root = os.path.join(os.path.dirname(__file__), "../lib/")
    node_class_paths = [os.path.join(lib_root, type) for type in [
        'dataio', 'dataprocess', 'flowcontrol', 'flowfunction', 'eventreceive', 'eventtransmit']]
    for node_root in node_class_paths:
        for node_name_file in os.listdir(node_root):
            if os.path.isfile(os.path.join(node_root, node_name_file)) and node_name_file != '__init__.py':
                node_name = node_name_file.replace('.py', '')
                if node_name not in lib_node_set:
                    lib_node_set.add(node_name)
                else:
                    raise Exception(
                        'Module or node repeat: {}'.format(node_name))
    if USER_LIB_PATH in kwargs:
        lib_path = kwargs[USER_LIB_PATH]
        if not os.path.isabs(lib_path):
            lib_path = os.path.join(os.getcwd(), lib_path)
        for path in glob.glob(lib_path, recursive=False):
            sys.path.append(path)
            for node_name_file in os.listdir(path):
                if os.path.isfile(os.path.join(path, node_name_file)) and node_name_file != '__init__.py':
                    node_name = node_name_file.replace('.py', '')
                    if node_name not in lib_node_set:
                        lib_node_set.add(node_name)
                    else:
                        raise Exception(
                            'Custom module or node repeat: {}'.format(node_name))
    return None


def put(key, value, **kwargs):
    with node_dict_lock:
        node_dict[key] = value


def get(key, **kwargs):
    ret = None
    with node_dict_lock:
        ret = node_dict[key]
    return ret


def has(key, **kwargs):
    ret = False
    with node_dict_lock:
        ret = key in node_dict
    return ret


def generate(key, **kwargs):
    """
    key: node_name, node's filename but bot include extension, module name
    """
    if key in lib_node_set:
        return imp.reload(globals.import_module(key))
    else:
        raise Exception('No "{}" module or node'.format(key))


def get_or_generate(id, node_name, **kwargs):
    """
    id: node_id or node_name, typically this is the node_id
    node_name: node_name, node's filename but bot include extension, also called the module name
    """
    ret = None
    with node_dict_lock:
        if id in node_dict:
            ret = node_dict[id]
    if ret is None:
        ret = generate(node_name)
    return ret


def get_or_generate_put(id, node_name, **kwargs):
    """
    id: node_id or node_name, typically this is the node_id
    node_name: node_name, node's filename but bot include extension, also called the module name
    """
    ret = None
    with node_dict_lock:
        if id in node_dict:
            ret = node_dict[id]
        else:
            ret = generate(node_name)
            node_dict[id] = ret
    return ret











def run(**kwargs):
    string_1=""
    string_2=""
    if "string_1" in kwargs:
        string_1=kwargs["string_1"]
    if "string_2" in kwargs:
        string_2=kwargs["string_2"]
    return {"result":string_1==string_2}





































# global

from importlib import import_module
import sys
import os
# or all platform. On window platform, sys.path[0] is same with basedir below:
basedir = os.path.dirname(os.path.abspath(__file__))
sys.path.insert(0, basedir)
sys.path.append(os.path.join(sys.path[0], 'lib/dataio'))
sys.path.append(os.path.join(sys.path[0], 'lib/dataprocess'))
sys.path.append(os.path.join(sys.path[0], 'lib/flowcontrol'))
sys.path.append(os.path.join(sys.path[0], 'lib/flowfunction'))
sys.path.append(os.path.join(sys.path[0], 'lib/eventreceive'))
sys.path.append(os.path.join(sys.path[0], 'lib/eventtransmit'))
sys.path.append(os.path.join(sys.path[0], '/system'))


# ====================================== global environment ======================================

ENV_WORK_DIR = os.getcwd()


def init_global_env(**kwargs):
    global ENV_WORK_DIR
    if 'ENV_WORK_DIR' in kwargs:
        ENV_WORK_DIR = kwargs['ENV_WORK_DIR']


# ====================================== data_center module ======================================

# data center
data_center = None


def init_data_center(**kwargs):
    global data_center
    DATA_CENTER_MODULE_NAME = "system.data_center"
    data_center = import_module(DATA_CENTER_MODULE_NAME)
    data_center.init(**kwargs)


# ====================================== node_center module ======================================


# node center
node_center = None


def init_node_center(**kwargs):
    global node_center
    NODE_CENTER_MODULE_NAME = "system.node_center"
    node_center = import_module(NODE_CENTER_MODULE_NAME)
    node_center.init(**kwargs)


# ====================================== log module ======================================


# log module
log_module = None
common_log = None
log = None


def init_log(**kwargs):
    """log module"""
    # get config and ini constant
    # global COMMON_LOG_FILE,LOG_FILE,LOG_MODULE_NAME
    LOG_MODULE_NAME = "system.log"
    # import module
    global log_module
    log_module = import_module(LOG_MODULE_NAME)
    # ini module
    log_module.init(**kwargs)
    # ini module function
    global common_log, log
    common_log = log_module.common_log
    log = log_module.log


# ====================================== aop module ======================================


# aop module
aop_module = None
aop = None


def init_aop(**kwargs):
    """aop module"""
    # get config and ini constant
    # global AOP_LOG_FILE,AOP_MODULE_NAME
    AOP_MODULE_NAME = "system.aop"
    # import module
    global aop_module
    aop_module = import_module(AOP_MODULE_NAME)
    # ini module
    aop_module.init(**kwargs)
    # ini module function
    global aop
    aop = aop_module.aop


# ====================================== engine module ======================================


# engine
engine = None


def init_engine(**kwargs):
    global engine
    ENGINE_MODULE_NAME = "system.engine"
    engine = import_module(ENGINE_MODULE_NAME)
    # params: graph:{node_list:[],edge_list:[]}
    engine.init(**kwargs)


# ====================================== init & start ======================================


def init(**kwargs):
    # print(kwargs)
    if "system" in kwargs and "global" in kwargs["system"]:
        init_global_env(**kwargs["system"]["global"])
    if "system" in kwargs and "data_center" in kwargs["system"]:
        init_data_center(**kwargs["system"]["data_center"])
    else:
        init_data_center()
    if "system" in kwargs and "node_center" in kwargs["system"]:
        init_node_center(**kwargs["system"]["node_center"])
    else:
        init_node_center()
    if "system" in kwargs and "aop" in kwargs["system"]:
        init_aop(**kwargs["system"]["aop"])
    else:
        init_aop()
    if "system" in kwargs and "log" in kwargs["system"]:
        init_log(**kwargs["system"]["log"])
    else:
        init_log()
    if "graph" in kwargs:
        init_engine(**kwargs["graph"])
    else:
        print("Error: Invalid profile!")


def start():
    engine.start()




import sys,os
import globals
import copy


def run(**kwargs):
    # print("control if...")
    bool=False
    if "bool" in kwargs:
        bool=kwargs["bool"]
    select_nxt_edge_index=0 if bool else 1
    node_config=kwargs["_config"]
    if "nxt_edge_id" not in node_config or len(node_config["nxt_edge_id"])<=select_nxt_edge_index:
        return
    edge_id=node_config["nxt_edge_id"][select_nxt_edge_index]
    if edge_id in globals.engine.edges_config:
        globals.engine.handle_flow_node(globals.engine.edges_config[edge_id]["nxt_id"])
    return {}
































def run(**kwargs):
    bool_1=False
    bool_2=False
    if "bool_1" in kwargs:
        bool_1=kwargs["bool_1"]
    if "bool_2" in kwargs:
        bool_2=kwargs["bool_2"]
    return {"result":bool_1 or bool_2}





































import globals
from threading import Thread


g_is_started=False
g_is_finished=False


def is_started():
    return g_is_started


def is_finished():
    return g_is_finished


def event_run(**kwargs):
    global g_is_started,g_is_finished
    g_is_started=True
    node_config=kwargs["_config"]
    if "nxt_edge_id" not in node_config or len(node_config["nxt_edge_id"])<=0:
        return
    edge_id=node_config["nxt_edge_id"][0]
    if edge_id in globals.engine.edges_config:
        globals.engine.handle_flow_node(globals.engine.edges_config[edge_id]["nxt_id"])
    g_is_finished=True
    return {}


def run(**kwargs):
    t=Thread(target=event_run,kwargs=kwargs)
    t.start()






import os
import sys
from setuptools import setup, find_packages
basedir = os.path.dirname(os.path.abspath(__file__))

long_description = None
with open(os.path.join(basedir, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

packages = []
packages.extend(["lib.{}".format(sub)
                for sub in find_packages(where='lib', include=['*'])])
packages.append('system')

setup(
    name='autorunx',
    version='1.1.2',
    author='Xuanfq',
    author_email='2624208682@qq.com',
    license='MIT',
    url='https://github.com/Xuanfq/AutoRunX-Python',
    description='AutoRunX is an open source low code framework.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=packages,
    py_modules=['autorunx', 'globals'],
    entry_points={
        'console_scripts': [
            'autorunx = autorunx:main',
            'arx = autorunx:main'
        ]
    },
    classifiers=[  # https://pypi.org/pypi?%3Aaction=list_classifiers
        # 发展时期,常见的如下
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',
        # 开发的目标用户
        'Intended Audience :: Developers',
        # 属于什么类型
        'Topic :: Software Development :: Build Tools',
        # 许可证信息
        'License :: OSI Approved :: MIT License',
        # 目标 Python 版本
        'Programming Language :: Python :: 3.9',
    ],
    keywords=['autorun', 'auto', 'run', 'automatic'],
)




import globals



def run(**kwargs):
    globals.log(msg="func test start...")
    params=""
    for k in kwargs:
        params="{}{}: {}".format("" if params=="" else params+", ",k,kwargs[k])
    globals.log(msg="params: { "+params+" }")
    globals.log(msg="func test end...")
    return {**kwargs}





























import sys
import os
import argparse
import globals
import json


parser = argparse.ArgumentParser()
parser.add_argument('-v', '--version',
                    action='version',
                    version='1.1.2',
                    help="checking version information"
                    )
parser.add_argument('-f', '--config-file',
                    required=True,
                    type=str,
                    help="specifies the configuration file to run")
parser.add_argument('-d', '--disable-config-file-system',
                    required=False,
                    action='store_true',
                    help="disables system setup from configuration file")
parser.add_argument('-l', '--system-node-lib-path',
                    required=False,
                    type=str,
                    help='set custom node library path')
parser.add_argument('-w', '--system-work-dir',
                    required=False,
                    type=str,
                    help='set working directory')
parser.add_argument('--system-aop-log-file',
                    required=False,
                    type=str,
                    help='set aop log file')
parser.add_argument('--system-aop-log-close',
                    required=False,
                    action='store_true',
                    help='set aop log close or not')
parser.add_argument('--system-log-common-log-file',
                    required=False,
                    type=str,
                    help='set common log file')
parser.add_argument('--system-log-function-log-file',
                    required=False,
                    type=str,
                    help='set function log file')


def main():
    args = parser.parse_args()
    if args.config_file is not None:
        config_path = args.config_file
        if not os.path.exists(config_path):
            print("Invalid parameter: {}".format(config_path))
            return
        config = None
        with open(config_path, "r", encoding="utf-8") as f:
            config = json.load(f)
        config['system'] = {'global':{}, 'node_center': {}, 'aop': {}, 'log': {}}
        if args.disable_config_file_system:
            config.pop('system', None)
        if args.system_work_dir is not None:
            config['system']['global']['ENV_WORK_DIR'] = args.system_work_dir
        if args.system_node_lib_path is not None:
            config['system']['node_center']['USER_LIB_PATH'] = args.system_node_lib_path
        if args.system_aop_log_file is not None:
            config['system']['aop']['AOP_LOG_FILE'] = args.system_aop_log_file
        if args.system_aop_log_close:
            config['system']['aop']['LOG_OPEN'] = False
        else:
            config['system']['aop']['LOG_OPEN'] = True
        if args.system_log_common_log_file is not None:
            config['system']['log']['COMMON_LOG_FILE'] = args.system_log_common_log_file
        if args.system_log_function_log_file is not None:
            config['system']['log']['LOG_FILE'] = args.system_log_function_log_file
        globals.init(**config)
        globals.start()
    else:
        parser.print_help()


def run(config):
    with open(config, "r", encoding="utf-8") as f:
        config = json.load(f)
        globals.init(**config)
        globals.start()


if __name__ == '__main__':
    main()




import sys
import os
import globals

import datetime


COMMON_LOG_FILE = "log/common.log"      # common log, all(log) in one.
LOG_FILE = "log/log.log"        # pure log, func log output


def init(**kwargs):
    global LOG_FILE, COMMON_LOG_FILE
    if "COMMON_LOG_FILE" in kwargs:
        COMMON_LOG_FILE = kwargs['COMMON_LOG_FILE']
    else:
        COMMON_LOG_FILE = os.path.join(globals.ENV_WORK_DIR, COMMON_LOG_FILE)
    if "LOG_FILE" in kwargs:
        LOG_FILE = kwargs['LOG_FILE']
    else:
        LOG_FILE = os.path.join(globals.ENV_WORK_DIR, LOG_FILE)
    for filefullpath in [COMMON_LOG_FILE, LOG_FILE]:
        if len(filefullpath.split('/')) > 1:
            [path, filename] = os.path.split(filefullpath)
            # print(path,filename)
            if not os.path.exists(path):
                os.makedirs(path)


def common_log(**kwargs):
    msg = kwargs['msg']
    # common log
    with open(COMMON_LOG_FILE, "+a", encoding="utf-8") as f:
        f.write(msg)
        # print(msg,end="")


def log(**kwargs):
    """
    kw:
    - msg: must, log message
    - second_log_file: optional
    """
    msg = kwargs['msg']
    msg = "[{}]: {}\n".format(datetime.datetime.now(), msg)
    print(msg, end="")
    # func log
    with open(LOG_FILE, "+a", encoding="utf-8") as f:
        f.write(msg)
    # common log
    common_log(msg=msg)
    # second log file
    if "second_log_file" in kwargs:
        if len(kwargs['second_log_file'].split('/')) > 1:
            [path, filename] = os.path.split(kwargs['second_log_file'])
            if not os.path.exists(path):
                os.makedirs(path)
        with open(kwargs['second_log_file'], "+a", encoding="utf-8") as f:
            f.write(msg)


# init(COMMON_LOG_FILE="log/common-log.log")
# log(msg="hello func!")
